# Example file to demonstrate the use of the VKOGA-PDE code:
# We use \Omega = [0, 1]^2, L = -LaPlace and the Gaussian kernel.


# Some imports
import numpy as np
from matplotlib import pyplot as plt

from kernels_PDE import Gaussian_laplace
from vkoga_PDE import VKOGA_PDE

np.random.seed(1)

# Create data set: Square domain
dim = 2

scale_domain = 1
X1 = scale_domain * np.random.rand(int(1e4), dim)
X2 = scale_domain * np.random.rand(400, dim)
X2[:100, 0] = 0
X2[100:200, 0] = scale_domain
X2[200:300, 1] = 0
X2[300:400, 1] = scale_domain

# Define some functions
u = lambda x: x[:, [1]] * x[:, [0]]**3
f = lambda x: -6 * x[:, [0]] * x[:, [1]]

# Define right hand side values
y1 = f(X1)
y2 = u(X2)

# Initialize and run models
kernel = Gaussian_laplace(dim=2)
maxIter = 200

model_f = VKOGA_PDE(kernel=kernel, greedy_type='f_greedy')
model_p = VKOGA_PDE(kernel=kernel, greedy_type='p_greedy')

_ = model_f.fit(X1, y1, X2, y2, maxIter=maxIter)
_ = model_p.fit(X1, y1, X2, y2, maxIter=maxIter)

# Plot some results
plt.figure(1)
plt.clf()
plt.plot(np.array(model_f.train_hist['f'])**.5, 'r')
plt.plot(np.array(model_p.train_hist['f'])**.5, 'b')
plt.yscale('log')
plt.xscale('log')
plt.legend(['f-greedy', 'p-greedy'])
plt.title('residuals')
plt.draw()

plt.figure(2)
plt.clf()
plt.plot(model_f.ctrs_[model_f.ind_ctrs1, 0], model_f.ctrs_[model_f.ind_ctrs1, 1], 'rx')
plt.plot(model_f.ctrs_[model_f.ind_ctrs2, 0], model_f.ctrs_[model_f.ind_ctrs2, 1], 'ro')
plt.title('Selected centers of f-greedy')
plt.draw()

plt.figure(3)
plt.clf()
plt.plot(model_p.ctrs_[model_p.ind_ctrs1, 0], model_p.ctrs_[model_p.ind_ctrs1, 1], 'bx')
plt.plot(model_p.ctrs_[model_p.ind_ctrs2, 0], model_p.ctrs_[model_p.ind_ctrs2, 1], 'bo')
plt.title('Selected centers of P-greedy')
plt.draw()

plt.figure(4)
plt.clf()
plt.plot(X1[:, 0], X1[:, 1], 'm.')
plt.plot(X2[:, 0], X2[:, 1], 'r.')


